@@ -1,13 +1,14 @@
 """
 
 """
+
 import logging
 import time
 from collections import Counter
 from unittest import mock
 
 import hiro
-from flask import Flask, request
+from flask import Flask, abort, make_response, request
 from werkzeug.exceptions import BadRequest
 
 from flask_limiter.constants import ConfigVars
@@ -50,6 +51,20 @@ def test_reset_unsupported(extension_factory, memcached_connection):
         assert "1 per 1 day" in cli.get("/").data.decode()
 
 
+def test_static_exempt(extension_factory):
+    app, limiter = extension_factory(default_limits=["1/minute"])
+
+    @app.route("/")
+    def root():
+        return "root"
+
+    with app.test_client() as cli:
+        assert cli.get("/").status_code == 200
+        assert cli.get("/").status_code == 429
+        assert cli.get("/static/image.png").status_code == 200
+        assert cli.get("/static/image.png").status_code == 200
+
+
 def test_combined_rate_limits(extension_factory):
     app, limiter = extension_factory(
         {ConfigVars.DEFAULT_LIMITS: "1 per hour; 10 per day"}
@@ -526,6 +541,63 @@ def test_application_shared_limit(extension_factory):
             assert 429 == cli.get("/t1").status_code
 
 
+def test_application_limit_conditional(extension_factory):
+    def app_limit_exempt():
+        return "X" in request.headers
+
+    def app_limit_deduct(response):
+        return response.status_code == 400
+
+    app, limiter = extension_factory(
+        application_limits=["2/minute"],
+        application_limits_exempt_when=app_limit_exempt,
+        application_limits_deduct_when=app_limit_deduct,
+    )
+
+    @app.route("/t1", methods=["GET", "POST"])
+    def t1():
+        return "route1"
+
+    @app.route("/t2")
+    def t2():
+        abort(400)
+
+    with hiro.Timeline().freeze():
+        with app.test_client() as cli:
+            assert 200 == cli.get("/t1").status_code
+            assert 400 == cli.get("/t2").status_code
+            assert 200 == cli.get("/t1").status_code
+            assert 400 == cli.get("/t2").status_code
+            assert 429 == cli.get("/t1").status_code
+            assert 429 == cli.get("/t2").status_code
+
+
+def test_application_limit_per_method(extension_factory):
+    app, limiter = extension_factory(
+        application_limits=["2/minute"],
+        application_limits_per_method=True,
+    )
+
+    @app.route("/t1", methods=["GET", "POST"])
+    def t1():
+        return "route1"
+
+    @app.route("/t2", methods=["GET", "POST"])
+    def t2():
+        return "route2"
+
+    with hiro.Timeline().freeze():
+        with app.test_client() as cli:
+            assert 200 == cli.get("/t1").status_code
+            assert 200 == cli.get("/t2").status_code
+            assert 429 == cli.get("/t1").status_code
+            assert 429 == cli.get("/t2").status_code
+            assert 200 == cli.post("/t1").status_code
+            assert 200 == cli.post("/t2").status_code
+            assert 429 == cli.post("/t1").status_code
+            assert 429 == cli.post("/t2").status_code
+
+
 def test_callable_default_limit(extension_factory):
     app, limiter = extension_factory(
         default_limits=[
@@ -823,3 +895,57 @@ def test_multiple_limiters_default_limits():
             assert cli.get("/test1").status_code == 429
             timeline.forward(50)
             assert cli.get("/test1").status_code == 200
+
+
+def test_meta_limits(extension_factory):
+    def meta_breach_cb(limit):
+        return make_response("Would you like some tea?", 429)
+
+    app, limiter = extension_factory(
+        default_limits=["2/second"],
+        meta_limits=["2/minute; 3/hour", lambda: "4/day"],
+        on_meta_breach=meta_breach_cb,
+        headers_enabled=True,
+    )
+
+    @app.route("/")
+    def root():
+        return "root"
+
+    with hiro.Timeline().freeze() as timeline:
+        with app.test_client() as cli:
+            for _ in range(2):
+                assert cli.get("/").status_code == 200
+                assert cli.get("/").status_code == 200
+                assert cli.get("/").status_code == 429
+                timeline.forward(1)
+
+            # blocked because of max 2 breaches/minute
+            assert cli.get("/").status_code == 429
+            timeline.forward(59)
+            assert cli.get("/").status_code == 200
+            assert cli.get("/").status_code == 200
+            assert cli.get("/").status_code == 429
+            timeline.forward(59)
+            # blocked because of max 3 breaches/hour
+            response = cli.get("/")
+            assert response.text == "Would you like some tea?"
+            assert response.status_code == 429
+            assert response.headers.get("X-RateLimit-Limit") == "3"
+            assert response.headers.get("X-RateLimit-Remaining") == "0"
+
+            # forward to 1 hour since start
+            timeline.forward(60 * 58)
+            assert cli.get("/").status_code == 200
+            assert cli.get("/").status_code == 200
+            assert cli.get("/").status_code == 429
+            # forward another hour and it should now be blocked for the day
+            timeline.forward(60 * 60)
+            response = cli.get("/")
+            assert response.status_code == 429
+            assert response.headers.get("X-RateLimit-Limit") == "4"
+            assert response.headers.get("X-RateLimit-Remaining") == "0"
+
+            # forward 22 hours
+            timeline.forward(60 * 60 * 22)
+            assert cli.get("/").status_code == 200
